@@ -93,14 +93,14 @@ class Session:
         try:
             login, t = self.db.getSession(sessionid)
         except Database.DatabaseInvalidSessionError:
-            self.log(Log.EVENT_INVALID_SESSIONID, request)
+            self.log.error("Invalid session identifier", request)
             raise AuthSessionInvalid()
 
         now = int(time.time())
 
         if now - t > self._expiration:
             self.db.deleteSession(sessionid)
-            self.log(Log.EVENT_SESSION_EXPIRED, request)
+            self.log.error("Session expired", request)
             raise AuthSessionExpired()
 
         self.db.updateSession(sessionid, now)
@@ -153,8 +153,5 @@ class LoginPasswordAuth(Auth, Session):
 
 
 class AnonymousAuth(Auth):
-    def __init__(self):
-        pass
-    
     def getUser(self, request):
-        return User.User("anonymous", User.ALL_PERMISSIONS)
+        return User.User(self.db, "anonymous", User.ALL_PERMISSIONS, self.db.getConfiguration("anonymous"))
