@@ -120,8 +120,7 @@ class Core:
         preludedb.preludedb_init()
         self._initDatabase()
         self._env.idmef_db = IDMEFDatabase.IDMEFDatabase(self._env.config.idmef_database)
-        self._env.auth = Auth.AnonymousAuth()
-        self._env.log = Log.Log()
+        self._env.log = Log.Log(self._env.config)
         self._initHostCommands()
         self._loadViews()
         self._loadModules()
@@ -178,15 +177,17 @@ class Core:
 
         if config.auth:
             self._env.auth = self._loadModule("auth", config.auth.name, config.auth)
-        
-        for backend in config.logs:
-            self._env.log.registerBackend(self._loadModule("log", backend.name, backend))
+        else:
+            self._env.auth = Auth.AnonymousAuth(self._env)
 
     def _setupView(self, view, request, parameters, user):
         object = view["object"]
         if not object.view_initialized:
             object.init(self._env)
             object.view_initialized = True
+        
+        object = copy.copy(object)
+
         object.request = request
         object.parameters = parameters
         object.user = user
@@ -194,13 +195,14 @@ class Core:
         object.env = self._env
         self._setupDataSet(object.dataset, request, user, view, parameters)
 
+        return object
+    
     def _cleanupView(self, view):
-        object = view["object"]
-        del object.request
-        del object.parameters
-        del object.user
-        del object.dataset
-        del object.env
+        del view.request
+        del view.parameters
+        del view.user
+        del view.dataset
+        del view.env
         
     def _setupDataSet(self, dataset, request, user, view=None, parameters={}):
         import prewikka.views
@@ -261,7 +263,7 @@ class Core:
     def _checkPermissions(self, request, view, user):
         if user and view.has_key("permissions"):
             if not user.has(view["permissions"]):
-                self._env.log(Log.EVENT_VIEW_FORBIDDEN, request, view, user)
+                self._env.log.warnings("Access to view forbidden", request, user)
                 raise User.PermissionDeniedError(user.login, view["name"])
 
     def _getParameters(self, request, view, user):
@@ -272,19 +274,18 @@ class Core:
         try:
             parameters.normalize(view["name"], user)
         except ParameterError, e:
-                self._env.log(Log.EVENT_INVALID_PARAMETERS, request, view, details=str(e))
-                raise InvalidQueryError(request.getQueryString())
+            self._env.log.error("%s" % str(e), request, user)
+            raise InvalidQueryError(request.getQueryString())
 
         return parameters
         
     def _getView(self, request, user):
-        name = request.arguments.get("view", "alert_listing")
-
+        name = request.getView()
         try:
             return self._views[name]
 
         except KeyError:
-            self._env.log(Log.EVENT_INVALID_VIEW, request=request, user=user)
+            self._env.log.error("View '%s' does not exist" % name, request=request, user=user)
             raise InvalidQueryError(request.getQueryString())
 
     def checkAuth(self, request):
@@ -297,25 +298,22 @@ class Core:
         return error.dataset, error.template
     
     def process(self, request):
-        self._env.log(Log.EVENT_QUERY, request)
-        
         try:
             user = None
             user = self.checkAuth(request)
             view = self._getView(request, user)
+
             self._checkPermissions(request, view, user)
             parameters = self._getParameters(request, view, user)
+            view_object = self._setupView(view, request, parameters, user)
 
-            self._setupView(view, request, parameters, user)
+            self._env.log.info("Loading view", request, user)
+            getattr(view_object, view["handler"])()
 
-            self._env.log(Log.EVENT_RENDER_VIEW, request, view, user)
-            
-            getattr(view["object"], view["handler"])()
-
-            dataset = view["object"].dataset
+            dataset = view_object.dataset
             template_name = view["template"]
 
-            self._cleanupView(view)
+            self._cleanupView(view_object)
             
         except Error.PrewikkaError, e:
             self._setupDataSet(e.dataset, request, user)
