@@ -25,6 +25,7 @@ import copy
 import re
 
 from prewikka import view, User, utils
+from prelude import idmef_path_new, idmef_path_get_value_type, IDMEF_VALUE_TYPE_STRING
 
 
 class _MyTime:
@@ -140,15 +141,14 @@ class MessageListingParameters(view.Parameters):
             
         if not self["timezone"] in ("frontend_localtime", "sensor_localtime", "utc"):
             raise view.InvalidValueError("timezone", self["timezone"])
-
+        
         # remove the bulshit
         try:
             del self["x"]
             del self["y"]
         except KeyError:
             pass
-
-
+        
 
 class AlertListingParameters(MessageListingParameters):
     allow_extra_parameters = True
@@ -278,13 +278,13 @@ class ListedMessage(dict):
         self.parameters = parameters
         self.timezone = parameters["timezone"]
 
-    def createInlineFilteredField(self, object, value, type=None):
+    def createInlineFilteredField(self, object, value, direction=None):
         if not value:
             return { "value": None, "inline_filter": None }
 
-        if type:
+        if direction:
             index = self.parameters.max_index
-            extra = { "%s_object_%d" % (type, index): object, "%s_value_%d" % (type, index): value }
+            extra = { "%s_object_%d" % (direction, index): object, "%s_value_%d" % (direction, index): value }
         else:
             extra = { object: value }
 
@@ -312,14 +312,15 @@ class ListedMessage(dict):
 
         return { "value": t }
 
-    def createHostField(self, object, value, type=None):
-        field = self.createInlineFilteredField(object, value, type)
+    def createHostField(self, object, value, category, direction=None):
+        field = self.createInlineFilteredField(object, value, direction)
         field["host_commands"] = [ ]
-        for command in "whois", "traceroute":
-            if self.env.host_commands.has_key(command):
-                field["host_commands"].append((command.capitalize(),
-                                               utils.create_link(command,
-                                                                 { "origin": self.view_name, "host": value })))
+        field["category"] = category
+        
+        for command in self.env.host_commands.keys():
+            field["host_commands"].append((command.capitalize(),
+                                           utils.create_link("Command",
+                                                             { "origin": self.view_name, "command": command, "host": value })))
 
         return field
     
@@ -346,8 +347,8 @@ class ListedHeartbeat(ListedMessage):
         self["node_addresses"] = [ ]
 
         for address in message["heartbeat.analyzer(-1).node.address"]:
-            self["node_addresses"].append(
-                self.createHostField("heartbeat.analyzer(-1).node.address.address", address["address"]))
+            hfield = self.createHostField("heartbeat.analyzer(-1).node.address.address", address["address"], address["category"])
+            self["node_addresses"].append(hfield)
             
         self["time"] = self.createTimeField(message["heartbeat.create_time"], self.parameters["timezone"])
 
@@ -361,44 +362,70 @@ class ListedSensorHeartbeat(ListedHeartbeat):
 class ListedAlert(ListedMessage):
     view_name = "alert_listing"
 
-    def _initValue(self, name, value):
-        import copy
-        
-        self["source"][name] = value
-        self["target"][name] = copy.copy(value)
+    def _getKnownValue(self, direction, key):
+        return { "alert.%s.interface" % direction: ("interface", None),
+                 "alert.%s.service.port" % direction: ("service", None),
+                 "alert.%s.process.name" % direction: ("process", None),
+                 "alert.%s.user.user_id.name" % direction: ("users", None),
+                 "alert.%s.file.path" % direction: ("files", self._setMessageDirectionTargetFile),
+                 "alert.%s.node.address.address" % direction: ("addresses", self._setMessageDirectionAddress),
+                 "alert.%s.node.name" % direction: ("addresses", self._setMessageDirectionNodeName), 
+                 }[key]
+    
+    def _initValue(self, dataset, name, value):
+        dataset[name] = value
 
+    def _initDirection(self, dataset):
+        self._initValue(dataset, "interface", { "value": None })
+        self._initValue(dataset, "protocol", { "value": None })
+        self._initValue(dataset, "service", { "value": None })
+        self._initValue(dataset, "process", { "value": None })
+        self._initValue(dataset, "users", [ ])
+        self._initValue(dataset, "files", [ ])
+        self._initValue(dataset, "addresses", [ ])
+
+        return dataset
+    
+    def _initDirectionIfNeeded(self, direction):
+        if len(self[direction]) == 0:        
+            self[direction].append(self._initDirection({ }))
+            
     def reset(self):
         self["sensors"] = [ ]    
         self["correlation_alert_name"] = None
-        self["source"] = { }
-        self["target"] = { }
-        self._initValue("interface", { "value": None })
-        self._initValue("protocol", { "value": None })
-        self._initValue("service", { "value": None })
-        self._initValue("process", { "value": None })
-        self._initValue("users", [ ])
-        self._initValue("files", [ ])
-        self._initValue("addresses", [ ])
-        self._initValue("empty", True)
+        self["source"] = [ ]
+        self["target"] = [ ]
         
     def __init__(self, *args, **kwargs):        
         apply(ListedMessage.__init__, (self, ) + args, kwargs)
         self.reset()
         
-    def _setMessageDirectionAddress(self, direction, address):
-        self[direction]["empty"] = False
-        self[direction]["addresses"].append(self.createHostField("alert.%s.node.address.address" % direction, address, type=direction))
+    def _setMessageDirectionAddress(self, direction, address, category=None):        
+        hfield = self.createHostField("alert.%s.node.address.address" % direction, address, category, direction=direction)
+        self._initDirectionIfNeeded(direction)
+        self[direction][-1]["addresses"].append(hfield)
 
-    def _setMessageDirectionNodeName(self, direction, name):
-        self[direction]["empty"] = False
-        self[direction]["addresses"].append(self.createHostField("alert.%s.node.name" % direction, name, type=direction))
+    def _setMessageDirectionNodeName(self, direction, name, extra=None):
+        self._initDirectionIfNeeded(direction)
+        self[direction][-1]["addresses"].append(self.createHostField("alert.%s.node.name" % direction, name, None, direction=direction))
 
-    def _setMessageDirectionGeneric(self, direction, object, value):
-        self[direction]["empty"] = False            
-        self[direction][object]["value"] = value
+    def _setMessageDirectionTargetFile(self, direction, path, extra=None):
+        self["target"][-1]["files"].append(self.createInlineFilteredField("alert.target.file.path", path, direction="target"))
         
-    def _setMessageDirection(self, dataset, message, direction):
-        empty = True
+    def _setMessageDirectionGeneric(self, direction, object, value, extra=None):
+        dset_name, function = self._getKnownValue(direction, object.replace("(0)", ""))        
+                        
+        self._initDirectionIfNeeded(direction)
+        if function:
+            function(direction, value, extra)
+        else:
+            if type(self[direction][-1][dset_name]) is list:
+                self[direction][-1][dset_name].append({ "value": value })
+            else:
+                self[direction][-1][dset_name]["value"] = value
+
+            
+    def _setMessageDirection(self, dataset, direction, obj):
         
         def set_main_and_extra_values(dataset, name, object_main, object_extra):
             if object_main != None:
@@ -407,84 +434,81 @@ class ListedAlert(ListedMessage):
             else:
                 dataset[name] = { "value": object_extra }
                 dataset[name + "_extra"] = { "value": None }
-
-            if dataset[name]["value"] != None:
-                empty = False
             
-        dataset["interface"] = { "value": message["alert.%s(0).interface" % direction] }
-
-        for userid in message["alert.%s(0).user.user_id" % direction]:
+        dataset["interface"] = { "value": obj["interface"] }
+        
+        for userid in obj["user.user_id"]:
             user = { }
-            empty = False
             dataset["users"].append(user)
             set_main_and_extra_values(user, "user", userid["name"], userid["number"])
 
-        name = message["alert.%s(0).node.name" % direction]
+        name = obj["node.name"]
         if name != None:
             self._setMessageDirectionNodeName(direction, name)
             
-        for addr in message["alert.%s(0).node.address" % direction]:
-            empty = False            
-            self._setMessageDirectionAddress(direction, addr["address"])
+        for addr in obj["node.address"]:
+            self._setMessageDirectionAddress(direction, addr["address"], addr["category"])
                         
-        set_main_and_extra_values(dataset, "process",
-                                  message["alert.%s(0).process.name" % direction],
-                                  message["alert.%s(0).process.pid" % direction])
+        set_main_and_extra_values(dataset, "process", obj["process.name"], obj["process.pid"])
 
         proto = None
-        if message["alert.%s(0).service.iana_protocol_name" % direction]:
-            proto = message["alert.%s(0).service.iana_protocol_name" % direction]
+        if obj["service.iana_protocol_name"]:
+            proto = obj["service.iana_protocol_name"]
             
-        elif message["alert.%s(0).service.iana_protocol_number" % direction]:
-            num = message["alert.%s(0).service.iana_protocol_number" % direction]
+        elif obj["service.iana_protocol_number"]:
+            num = obj["service.iana_protocol_number"]
             proto = utils.protocol_number_to_name(num)
 
         if not proto:
-            proto = message["alert.%s(0).service.protocol" % direction]
+            proto = obj["service.protocol"]
        
         set_main_and_extra_values(dataset, "protocol", proto, None)
-        set_main_and_extra_values(dataset, "service", message["alert.%s(0).service.port" % direction], None)
+        set_main_and_extra_values(dataset, "service", obj["service.port"], None)
 
         dataset["files"] = []
-        dataset["empty"] = empty
 
     def setMessageSource(self, message):
-        self._setMessageDirection(self["source"], message, "source")
-
+        for source in message["alert.source"]:
+            dataset = { }
+            self["source"].append(dataset)
+            
+            self._initDirection(dataset)
+            self._setMessageDirection(dataset, "source", source)
+            
     def setMessageTarget(self, message):
-        self._setMessageDirection(self["target"], message, "target")
-
-        flist = []
-        empty = self["target"]["empty"]
         
-        for f in message["alert.target(0).file"]:
-            if f["path"] in flist:
-                continue
+        for target in message["alert.target"]:
+            dataset = { }
+            self["target"].append(dataset)
+            
+            self._initDirection(dataset)
+            self._setMessageDirection(dataset, "target", target)
 
-            empty = False
-            flist.append(f["path"])
-            self["target"]["files"].append(self.createInlineFilteredField("alert.target.file.path",
-                                                                          f["path"], type="target"))
-
-        self["target"]["empty"] = empty
+            flist = []
+            for f in target["file"]:
+                if f["path"] in flist:
+                    continue
+                
+                flist.append(f["path"])
+                self._setMessageDirectionTargetFile("target", f["path"])
         
 
     def setMessageClassificationReferences(self, dataset, message):
         urls = [ ]
-
+        
+        val = self.env.config.general.getOptionValue("external_link_new_window", "true")
+        if (not val and self.env.config.general.has_key("external_link_new_window")) or \
+           (val == None or val.lower() in ["true", "yes"]):
+            external_link_target = "_blank"
+        else:
+            external_link_target = "_self"
+        
         for ref in message["alert.classification.reference"]:
             fstr = ""
 
             url = ref["url"]
-            if url:
-                external_link_new_window = self.env.config.general.getOptionValue("external_link_new_window", "true")
-                if (not external_link_new_window and self.env.config.general.has_key("external_link_new_window")) or \
-                   (external_link_new_window == None or external_link_new_window.lower() in [ "true", "yes" ]):
-                    target = "_blank"
-                else:
-                    target = "_self"
-                    
-                fstr="<a target='%s' href='%s'>" % (target, url)
+            if url:                    
+                fstr="<a target='%s' href='%s'>" % (external_link_target, url)
 
             origin = ref["origin"]
             if origin:
@@ -508,7 +532,7 @@ class ListedAlert(ListedMessage):
         self.setMessageClassificationReferences(dataset, message)
         dataset["classification"] = self.createInlineFilteredField("alert.classification.text",
                                                                    message["alert.classification.text"])
-
+        
     def setMessageCorrelationAlertInfo(self, dataset, message, ident):
         fetch_source_info=True
         fetch_target_info=True
@@ -597,7 +621,7 @@ class ListedAlert(ListedMessage):
     def addSensor(self, name, node_name):
         sensor = { }
         self["sensors"].append(sensor)
-        sensor["name"] = self.createInlineFilteredField("alert.analyzer.name", name, type="analyzer")
+        sensor["name"] = self.createInlineFilteredField("alert.analyzer.name", name, direction="analyzer")
         sensor["node_name"] = { "value": node_name }
         
     def setMessageTime(self, message):
@@ -620,8 +644,7 @@ class ListedAlert(ListedMessage):
         self.addSensor(message["alert.analyzer(-1).name"], message["alert.analyzer(-1).node.name"])
         self.setMessageTime(message)
         self.setMessageInfo(message, ident)
-
-
+        
 class ListedSensorAlert(ListedAlert):
     view_name = "sensor_alert_listing"
 
@@ -633,7 +656,9 @@ class ListedAggregatedAlert(ListedAlert):
         self["aggregated"] = True
         self["aggregated_classification_hidden"] = 0
         self["infos"] = [ ]
-        
+        self["source"] = [ ]
+        self["target"] = [ ]
+                    
     def setTime(self, time_min, time_max):
         self["time_min"] = self.createTimeField(time_min, self.parameters["timezone"])
         self["time_max"] = self.createTimeField(time_max, self.parameters["timezone"])
@@ -904,7 +929,7 @@ class AlertListing(MessageListing, view.View):
 
             if newcrit:
                 criteria.append(newcrit)
-
+            
             self.dataset[type] = self.parameters[type]
             self.dataset["%s_filtered" % type] = True
         else:
@@ -941,17 +966,10 @@ class AlertListing(MessageListing, view.View):
         else:                
             return False
                 
-    def _setAggregatedMessagesNoValues(self, criteria, aggregated_on):
-        filter_on = []
-        filter_values = []
+    def _setAggregatedMessagesNoValues(self, criteria, ag_s, ag_t, ag_c):
         atomic_ignore_list = []
-
-        for column in "source", "target", "classification":
-            if len(self.parameters[column]):
-                for item in self.parameters[column]:
-                      filter_on.append(item[0])
-                      filter_values.append(item[1])
-
+        ag_list = ag_s + ag_t + ag_c
+        
         selection = [ "alert.messageid", "alert.create_time" ]
         results2 = self.env.idmef_db.getValues(selection, criteria + ["alert.correlation_alert.name"])
 
@@ -960,37 +978,37 @@ class AlertListing(MessageListing, view.View):
             ca_ident = self.env.idmef_db.getAlertIdents(criteria + [ "alert.messageid = %s" % row[0] ], 1, -1)[0]
             message = self.env.idmef_db.getAlert(ca_ident)
             self._ignoreAtomicIfNeeded(message, atomic_ignore_list)
-            results += [ [ row[0] ] + [None for i in aggregated_on] + [ca_ident] + [message] + [ row[1] ] ]
-
+            results += [ [ row[0] ] + [None for i in ag_list] + [ca_ident] + [message] + [ row[1] ] ]
+            
         ignore_criteria = [ ]
         for messageid in atomic_ignore_list:
             ignore_criteria += [ "alert.messageid != '%s'" % messageid ]
             
         ##
-        selection = [ "%s/group_by" % path for path in aggregated_on ] + \
+        selection = [ "%s/group_by" % path for path in ag_list ] + \
                     [ "count(alert.create_time)", "max(alert.create_time)/order_desc" ]
 
         results += self.env.idmef_db.getValues(selection, criteria + [ "! alert.correlation_alert.name"] + ignore_criteria)
         results.sort(lambda x, y: int(int(y[-1]) - int(x[-1])))
         total_results = len(results)
-            
+           
         for values in results[self.parameters["offset"]:self.parameters["offset"]+self.parameters["limit"]]:
             start = 0
             aggregated_source_values = []
             aggregated_target_values = []
             aggregated_classification_values = []
+            
+            if len(ag_s) > 0:
+                start = len(ag_s)
+                aggregated_source_values = values[:len(ag_s)]
 
-            if self.parameters["aggregated_source"] != ["none"]:
-                start = len(self.parameters["aggregated_source"])
-                aggregated_source_values = values[:len(self.parameters["aggregated_source"])]
-
-            if self.parameters["aggregated_target"] != ["none"]:
-                last = start + len(self.parameters["aggregated_target"])
+            if len(ag_t) > 0:
+                last = start + len(ag_t)
                 aggregated_target_values = values[start:last]
                 start = last
 
-            if self.parameters["aggregated_classification"] != ["none"]:
-                last = start + len(self.parameters["aggregated_classification"])
+            if len(ag_c) > 0:
+                last = start + len(ag_c)
                 if values[start:last]:
                     aggregated_classification_values = values[start:last]
                 start = last
@@ -1006,29 +1024,33 @@ class AlertListing(MessageListing, view.View):
             delete_criteria = [ ]
             message = self.listed_aggregated_alert(self.env, self.parameters)
 
-                
-            for path, value in zip(aggregated_on, values[:start]):
+            for path, value in zip(ag_list, values[:start]):
+                                
                 if path.find("source") != -1:
                     direction = "source"
-                else:
+                    value_category = aggregated_source_values[-1]
+                elif path.find("target") != -1:
                     direction = "target"
-
+                    value_category = aggregated_target_values[-1]
+                else:
+                    direction = None
+                    
+                if re.compile("alert.%s(\([0-9\*]*\))?\.node\.address(\([0-9\*]*\))?\.category" % direction).match(path):
+                    continue
+                
                 if not value:
-                    criterion = "(! %s || %s == '')" % (path, path)
-
+                    if idmef_path_get_value_type(idmef_path_new(path), -1) != IDMEF_VALUE_TYPE_STRING:
+                        criterion = "! %s" % (path)
+                    else:
+                        criterion = "(! %s || %s == '')" % (path, path)
                 else:
                     criterion = "%s == '%s'" % (path, utils.escape_criteria(str(value)))
-                    
-                    if path.find("address") != -1 or path.find("node.name") != -1:
-                        message._setMessageDirectionAddress(direction, value)
-                    
-                    for var in [ ("user", None), ("process", None), ("service", None), ("port", "service") ]:
-                        if path.find(var[0]) != -1:
-                            message._setMessageDirectionGeneric(direction, var[1] or var[0], value)
-               
+                    if direction in ("source", "target"):
+                        message._setMessageDirectionGeneric(direction, path, value, value_category)
+                       
                 criteria2.append(criterion)
                 delete_criteria.append(criterion)
-            
+
             time_min = self.env.idmef_db.getValues(["alert.create_time/order_asc"], criteria2, limit=1)[0][0]
             time_max = self.env.idmef_db.getValues(["alert.create_time/order_desc"], criteria2, limit=1)[0][0]
             
@@ -1041,19 +1063,19 @@ class AlertListing(MessageListing, view.View):
             message.setTime(time_min, time_max)
             message.setCriteriaForDeletion(delete_criteria)
 
-            results = self.env.idmef_db.getValues(["alert.analyzer(-1).name/group_by",
-                                                   "alert.analyzer(-1).node.name/group_by"],
-                                                  criteria2)
+            res = self.env.idmef_db.getValues(["alert.analyzer(-1).name/group_by",
+                                               "alert.analyzer(-1).node.name/group_by"],
+                                              criteria2)
 
-            for analyzer_name, analyzer_node_name in results:
+            for analyzer_name, analyzer_node_name in res:
                 message.addSensor(analyzer_name, analyzer_node_name)
 
-            results = self.env.idmef_db.getValues(["alert.classification.text/group_by",
-                                                   "alert.assessment.impact.severity/group_by",
-                                                   "alert.assessment.impact.completion/group_by",
-                                                   "count(alert.create_time)"], criteria2 + ignore_criteria)
+            res = self.env.idmef_db.getValues(["alert.classification.text/group_by",
+                                               "alert.assessment.impact.severity/group_by",
+                                               "alert.assessment.impact.completion/group_by",
+                                               "count(alert.create_time)"], criteria2 + ignore_criteria)
                 
-            results.sort(lambda x, y: cmp_severities(x[1], y[1]))
+            res.sort(lambda x, y: cmp_severities(x[1], y[1]))
 
             parameters = self._createAggregationParameters(aggregated_classification_values,
                                                            aggregated_source_values, aggregated_target_values)
@@ -1069,14 +1091,14 @@ class AlertListing(MessageListing, view.View):
                                                                                     { "aggregated_classification":
                                                                                       "alert.classification.text" } )
 
-            if len(results[:self._max_aggregated_classifications]) > 1:
+            if len(res[:self._max_aggregated_classifications]) > 1:
                 classification = None
             else:
-                classification = results[0][0] or ""
+                classification = res[0][0] or ""
 
             result_count = 0
 
-            for classification, severity, completion, count in results:
+            for classification, severity, completion, count in res:
                 if result_count >= self._max_aggregated_classifications:
                     result_count += 1
                     continue
@@ -1144,7 +1166,7 @@ class AlertListing(MessageListing, view.View):
             
         i = self.parameters.max_index
         for path, value in zip(self.parameters["aggregated_source"], aggregated_source_values):
-            if value == None:
+            if value == None or path == "alert.source(0).node.address(0).category":
                 continue
             
             parameters["source_object_%d" % i] = path.replace("(0)", "")
@@ -1153,7 +1175,7 @@ class AlertListing(MessageListing, view.View):
         
         i = self.parameters.max_index
         for path, value in zip(self.parameters["aggregated_target"], aggregated_target_values):
-            if value == None:
+            if value == None or path == "alert.target(0).node.address(0).category":
                 continue
             
             parameters["target_object_%d" % i] = path.replace("(0)", "")
@@ -1164,24 +1186,35 @@ class AlertListing(MessageListing, view.View):
     
     def _setMessages(self, criteria):
         self.dataset["messages"] = [ ]
-
         self.dataset["aggregated_source"] = self.parameters["aggregated_source"]
         self.dataset["aggregated_target"] = self.parameters["aggregated_target"]
         self.dataset["aggregated_classification"] = self.parameters["aggregated_classification"]
+        
+        if "alert.source(0).node.address(0).address" in self.parameters["aggregated_source"]:
+            self.parameters["aggregated_source"] += [ "alert.source(0).node.address(0).category" ]
 
-        aggregated_on = []
-        if self.parameters["aggregated_source"] != ["none"]:
-            aggregated_on += self.parameters["aggregated_source"]
+        if "alert.target(0).node.address(0).address" in self.parameters["aggregated_target"]:
+            self.parameters["aggregated_target"] += [ "alert.target(0).node.address(0).category" ]
 
-        if self.parameters["aggregated_target"] != ["none"]:
-            aggregated_on += self.parameters["aggregated_target"]
+        ag_s = self.parameters["aggregated_source"][:]
+        ag_t = self.parameters["aggregated_target"][:]
+        ag_c = self.parameters["aggregated_classification"][:]
 
-        if self.parameters["aggregated_classification"] != ["none"]:
-            aggregated_on += self.parameters["aggregated_classification"]
+        for l in ag_s, ag_t, ag_c:
+            while "none" in l:
+                l.remove("none")
 
-        if len(aggregated_on) > 0:
-            return self._setAggregatedMessagesNoValues(criteria, aggregated_on)
+        if len(ag_s + ag_t + ag_c) > 0:
+            ret = self._setAggregatedMessagesNoValues(criteria, ag_s, ag_t, ag_c)
 
+            try: self.parameters["aggregated_source"].remove("alert.source(0).node.address(0).category")
+            except: pass
+            
+            try: self.parameters["aggregated_target"].remove("alert.target(0).node.address(0).category")
+            except: pass
+            
+            return ret
+        
         atomic_ignore_list = []
         for ident in self.env.idmef_db.getAlertIdents(criteria, self.parameters["limit"], self.parameters["offset"]):
             message = self.env.idmef_db.getAlert(ident)
@@ -1199,12 +1232,12 @@ class AlertListing(MessageListing, view.View):
     def _setDatasetConstants(self):
         self.dataset["available_aggregations"] = { }
         self.dataset["available_aggregations"]["classification"] = ( ("", "none"),
-                                                                     ("text", "alert.classification.text"))
+                                                                     ("classification", "alert.classification.text"))
         
         for category in "source", "target":
             tmp = (("", "none"),
                    ("address", "alert.%s(0).node.address(0).address" % category),
-                   ("name", "alert.%s(0).node.name" % category),
+                   ("node name", "alert.%s(0).node.name" % category),
                    ("user", "alert.%s(0).user.user_id(0).name" % category),
                    ("process", "alert.%s(0).process.name" % category),
                    ("service", "alert.%s(0).service.name" % category),
@@ -1246,6 +1279,7 @@ class AlertListing(MessageListing, view.View):
         self.dataset["limit"] = self.parameters["limit"]
         self.dataset["total"] = total
 
+
         self._setNavNext(self.parameters["offset"], total)
         self._setTimezone()
 
@@ -1307,7 +1341,7 @@ class HeartbeatListing(MessageListing, view.View):
         self._deleteMessages()
         criteria = [ ]
         start = end = None
-        
+
         if self.parameters.has_key("timeline_unit") and self.parameters["timeline_unit"] != "unlimited":
             start, end = self._getTimelineRange()
             criteria.append("heartbeat.create_time >= '%s' && heartbeat.create_time < '%s'" % (str(start), str(end)))
